from pydantic import Field

from models.base import TaranisBaseModel
from models.assess import StoryTag


class Dashboard(TaranisBaseModel):
    _core_endpoint = "/dashboard"
    _model_name = "dashboard"
    _pretty_name = "Dashboard"
    _cache_timeout = 30
    total_news_items: int = 0
    total_products: int = 0
    report_items_completed: int = 0
    report_items_in_progress: int = 0
    total_database_items: int = 0
    latest_collected: str | None = None
    schedule_length: int | None = None
    conflict_count: int | None = None


class TrendingCluster(TaranisBaseModel):
    _core_endpoint = "/dashboard/trending-clusters"
    _model_name = "trending_clusters"
    _pretty_name = "Trending Clusters"

    name: str
    tags: list[StoryTag] = Field(default_factory=list)
    size: int | None = None



from datetime import datetime
from pydantic import Field

from models.base import TaranisBaseModel


class WorkerProduct(TaranisBaseModel):
    data: bytes | None = None
    mime_type: str | None = None


class Product(TaranisBaseModel):
    id: str | None = None
    title: str
    description: str | None = None
    created: datetime | None = None
    auto_publish: bool | None = None
    product_type_id: int
    report_items: list[str] = Field(default_factory=list)
    last_rendered: datetime | None = None



from models.admin import ReportItemAttribute
from datetime import datetime
from pydantic import Field

from models.base import TaranisBaseModel


class ReportItemCpe(TaranisBaseModel):
    id: int | None = None
    value: str


class ReportItem(TaranisBaseModel):
    id: str
    title: str | None = None
    created: datetime | None = None
    last_updated: datetime | None = None
    completed: bool | None = None
    user_id: int | None = None
    report_item_type_id: int | None = None
    stories: list[str] = Field(default_factory=list)
    attributes: list[ReportItemAttribute] = Field(default_factory=list)
    report_item_cpes: list[ReportItemCpe] = Field(default_factory=list)



from datetime import datetime
from pydantic import field_validator, model_validator, field_serializer
import langcodes

from models.base import TaranisBaseModel


class NewsItem(TaranisBaseModel):
    _core_endpoint = "/assess/newsitems"

    osint_source_id: str
    hash: str = ""
    author: str = ""
    title: str = ""
    review: str | None = None
    content: str = ""
    web_url: str = ""
    source: str = ""
    published_date: datetime | None = None
    collected_date: datetime | None = None
    attributes: list[str] | None = None
    language: str | None = None

    @field_validator("language", mode="before")
    def normalize_language_code(cls, v):
        if not v:
            return None
        try:
            lang = langcodes.find(v)
            return lang.language
        except LookupError:
            return None

    @model_validator(mode="after")
    def set_default_dates(self):
        now = datetime.now()
        if not self.collected_date:
            self.collected_date = now
        if not self.published_date:
            self.published_date = self.collected_date
        return self

    @field_serializer("published_date", "collected_date", when_used="always")
    def serialize_datetime(self, value: datetime | None) -> str | None:
        return value.isoformat() if value else None


class StoryTag(TaranisBaseModel):
    name: str
    size: int
    type: str | None = None



from pydantic import BaseModel
from typing import ClassVar, TypeVar

T = TypeVar("T", bound="TaranisBaseModel")


class TaranisBaseModel(BaseModel):
    _core_endpoint: ClassVar[str]
    _cache_timeout: ClassVar[int]
    _model_name: ClassVar[str] = ""
    _pretty_name: ClassVar[str] = ""

    def model_dump(self, *args, **kwargs):
        kwargs.setdefault("exclude_none", True)
        kwargs.setdefault("exclude_defaults", False)
        kwargs.setdefault("exclude_unset", False)
        return super().model_dump(*args, **kwargs)



from pydantic import BaseModel


class PagingData(BaseModel):
    page: int | None = None
    limit: int | None = None
    order: str | None = None
    search: str | None = None


class CacheObject(list):
    def __init__(self, iterable=None, page: int = 1, limit: int = 20, order: str = "", length: int | None = None):
        iterable = iterable or []
        super().__init__(iterable)
        self.page = page
        self.limit = limit
        self.order = order
        self.length = length or len(iterable)

    def __getitem__(self, item):
        result = super().__getitem__(item)
        if isinstance(item, slice):
            return CacheObject(result, page=self.page, limit=self.limit, order=self.order, length=self.length)
        return result

    @property
    def current_page(self):
        return self.page

    @property
    def offset(self):
        return (self.page - 1) * self.limit

    @property
    def total_pages(self):
        return (self.length + self.limit - 1) // self.limit

    @property
    def last_page(self) -> bool:
        return self.page == self.total_pages

    @property
    def first_page(self) -> bool:
        return self.page == 1

    @property
    def current_range(self):
        return f"{self.offset + 1}-{min(self.offset + self.limit, self.length)}"

    def search(self, search: str) -> "CacheObject":
        result_object: CacheObject = CacheObject([])
        for co in self:
            for field in co._search_fields:
                value = getattr(co, field, "")
                if search.lower() in str(value).lower():
                    result_object.append(co)
                    break
        return result_object

    def search_and_paginate(self, paging: PagingData | None) -> "CacheObject":
        if not paging:
            return self
        if paging.search:
            return self.search(paging.search).paginate(paging)
        return self.paginate(paging)

    def paginate(self, paging: PagingData | None) -> "CacheObject":
        if not paging:
            return self[self.offset : self.offset + self.limit]
        if paging.order:
            sort_key, direction = paging.order.split("_")
            self.sort(key=lambda x: getattr(x, sort_key), reverse=direction == "desc")
        if paging.page:
            self.page = paging.page
        if paging.limit:
            self.limit = paging.limit

        return self[self.offset : self.offset + self.limit]
